/********************************************/
/* Traduccin de archivos                   */
/*                                          */
/* AUTOR: Ricardo Ferris, Jesus Albert      */
/* FECHA: 5 de marzo de 2004                */
/*                                          */
/* PROPOSITO: Traducir archivos escritos    */
/* usando pseudo etiquetas html en          */
/* archivos html de verdad                  */
/********************************************/

#include <iostream>
#include <fstream>
#include <string>
#include <stdlib.h>
using namespace std;

const int MAX = 200; //numero maximo de etiquetas posibles

/*
* DECLARACION DE TIPOS
*/

//Definicion de una entrada de la tabla de equivalencias entre etiquetas
struct Equivalencia
{
    string html;    //etiqueta HTML
    string normal;  //etiqueta castellano/valenciano
};

typedef Equivalencia Vector[MAX];

//Tabla de equivalencias entre etiquetas
struct Tabla
{
    Vector info;    //Vector de equivalencias
    int num;        //posiciones ocupadas del vector
};

/*
* DECLARACION DE FUNCIONES EMPLEADAS
*/

/*
* Lectura de la tabla de equivalencias desde el archivo
* se asume que el nombre del archivo es fijo "tabla.txt"
*/
void LeerTabla (Tabla & tab);

/*
* Ordenacin de la tabla de equivalencias para poder buscar
* con el mtodo de busqueda binaria
*/
void Ordenar (Tabla & tab);

/*
* Esquema general de traduccin
* El fichero de entrada f, se traduce en el fichero
* de salida g, segun la tabla de equivalencias tab
*/
void Traducir (ifstream & f, ofstream & g, Tabla tab);


/*
 * Leer una cadena que puede ser una etiqueta de formato
 * Devuelve una etiqueta valida (si ha llegado al simbolo '>'
 * o no valida (si ha encontrado un espacio en blanco o el final
 * del fichero.
 * Se invoca cuando se lee "<" en el archivo original
 */
string LeerEtiqueta (ifstream & f);

/*
* Traducir una etiqueta en su equivalente HTML
* segun las equivalencias almacenadas en la tabla.
* Si no se encuentra la etiqueta en la tabla, entonces
* se devuelve la etiqueta original.
* Implementa el algoritmo de bsqueda binaria
*/
string TraduceEtiqueta (string etiqueta, Tabla tab);

/********************************************/
/*                                          */
/*                  MAIN()                  */
/*                                          */
/********************************************/
int main()
{
    Tabla tab;
    string nom_fich_in, nom_fich_out;
    ifstream f_in;
    ofstream f_out;

    //Cargar tablas desde el archivo
    LeerTabla (tab);

    if (tab.num == 0)
    {
        cout << "No hay datos en la tabla de equivalencias.\n";
    }
    else
    {
        //Ordenar la tabla antes de traducir
        Ordenar (tab);
        //Entrada de datos (archivos)
        cout << "Nombre del fichero para traducir (txt): ";
        getline (cin, nom_fich_in);
        cout << "Nombre del fichero de salida (html): ";
        getline (cin, nom_fich_out);

        f_in.open (nom_fich_in.c_str () );
        f_out.open (nom_fich_out.c_str () );

        if (!f_in || !f_out)
            cout << "Error abriendo ficheros";
        else
            Traducir (f_in, f_out, tab);

        f_in.close ();
        f_out.close ();
    }

      system("PAUSE");
      return 0;
}

/*
* Esquema general de traduccin
* El fichero de entrada f, se traduce en el fichero
* de salida g, segun la tabla de equivalencias tab
*/
void Traducir (ifstream & f, ofstream & g, Tabla tab)
{
    char c;
    string etiqueta, traduccion;
    bool bien;

    c = f.get ();
    while (!f.eof () )
    {
        if (c == '<')
        {
            etiqueta = LeerEtiqueta (f);
            traduccion = TraduceEtiqueta (etiqueta, tab);

            g << traduccion;
        }
        else
            g << c;

        c = f.get ();
    }
}

/*
* Lectura de la tabla de equivalencias desde el archivo
* se asume que el nombre del archivo es fijo "tabla.txt"
*/
void LeerTabla (Tabla & tab)
{
    ifstream f;
    int i = 0;

    f.open ("tabla.txt");

    if (!f)
        cout << "Error abriendo fichero de tabla.\n";
    else
    {
        while (f >> tab.info[i].html)
            f >> tab.info[i++].normal;

        f.close ();
    }
    tab.num = i;

    return;
}

/*
* Ordenacin de la tabla de equivalencias para poder buscar
* con el mtodo de busqueda binaria.
* Se ordena segun la etiqueta normal
*/
void Ordenar (Tabla & tab)
{
    int i, j;
    string aux;

    for (i = tab.num - 2; i >= 0; i--)
    {
        tab.info[tab.num] = tab.info[i];
        j = i + 1;
        while (tab.info[tab.num].normal > tab.info[j].normal)
        {
            tab.info[j - 1] = tab.info[j];
            j++;
        }
        tab.info[j - 1] = tab.info[tab.num];
    }
}


/*
 * Leer una cadena que puede ser una etiqueta de formato
 * Devuelve una etiqueta valida (si ha llegado al simbolo '>'
 * o no valida (si ha encontrado un espacio en blanco o el final
 * del fichero.
 * Se invoca cuando se lee "<" en el archivo original
 */
string LeerEtiqueta (ifstream & f)
{
    char c;
    string etiqueta;

    //marca de inicio de etiqueta
    etiqueta = "<";

    c = f.get ();
    while (!f.eof () && c != ' ' && c != '>')
    {
        etiqueta += c;
        c = f.get ();
    }

    if (!f.eof () )
        etiqueta += c;

    return etiqueta;
}

/*
* Traducir una etiqueta en su equivalente HTML
* segun las equivalencias almacenadas en la tabla.
* Si no se encuentra la etiqueta en la tabla, entonces
* se devuelve la etiqueta original.
* Implementa el algoritmo de bsqueda binaria
*/
string TraduceEtiqueta (string etiqueta, Tabla tab)
{
    int izq, der, cen;
    string traduccion;

    izq = 0;
    der = tab.num - 1;
    cen = (izq + der) / 2;

    while ( (izq <= der) && (etiqueta != tab.info[cen].normal) )
    {
        if (etiqueta < tab.info[cen].normal)
            der = cen - 1;
        else
            izq = cen + 1;
        cen = (izq + der) / 2;
    }

    if (der < izq)  //la etiqueta no se encuentra
        traduccion = etiqueta;  //no hay traduccion
    else
        traduccion = tab.info[cen].html;    //traduccion

    return traduccion;
}

